from __future__ import absolute_import, unicode_literals

from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext_lazy as _

from .models import ENABLED, Plugin
from .models import PluginPoint as PluginPointModel
from .utils import db_table_exists, get_plugin_name

_PLUGIN_POINT = "<class 'djangoplugins.point.PluginPoint'>"


def is_plugin_point(cls):
    return repr(cls.__base__) == _PLUGIN_POINT


class PluginMount(type):
    """
    See: http://martyalchin.com/2008/jan/10/simple-plugin-framework/

    """

    points = []

    def __new__(meta, class_name, bases, class_dict):
        cls = type.__new__(meta, class_name, bases, class_dict)
        if is_plugin_point(cls):
            PluginMount.points.append(cls)
        return cls

    def __init__(cls, name, bases, attrs):
        if is_plugin_point(cls):
            # This branch only executes when processing the mount point itself.
            # So, since this is a new plugin type, not an implementation, this
            # class shouldn't be registered as a plugin. Instead, it sets up a
            # list where plugins can be registered later.
            cls.plugins = []
        elif hasattr(cls, "plugins"):
            # This must be a plugin implementation, which should be registered.
            # Simply appending it to the list is all that's needed to keep
            # track of it later.
            cls.plugins.append(cls)

    DoesNotExist = ObjectDoesNotExist


class PluginPoint(metaclass=PluginMount):
    @classmethod
    def get_pythonpath(cls):
        return get_plugin_name(cls)

    @classmethod
    def is_active(cls):
        if is_plugin_point(cls):
            raise Exception(_("This method is only available to plugin " "classes."))
        else:
            return cls.get_model().is_active()

    @classmethod
    def get_model(cls, name=None, status=ENABLED):
        """
        Returns model instance of plugin point or plugin, depending from which
        class this methos is called.

        Example::

            plugin_model_instance = MyPlugin.get_model()
            plugin_model_instance = MyPluginPoint.get_model('plugin-name')
            plugin_point_model_instance = MyPluginPoint.get_model()

        """
        ppath = cls.get_pythonpath()
        if is_plugin_point(cls):
            if name is not None:
                kwargs = {}
                if status is not None:
                    kwargs["status"] = status
                return Plugin.objects.get(point__pythonpath=ppath, name=name, **kwargs)
            else:
                return PluginPointModel.objects.get(pythonpath=ppath)
        else:
            return Plugin.objects.get(pythonpath=ppath)

    @classmethod
    def get_plugin(cls, name=None, status=ENABLED):
        return cls.get_model(name, status).get_plugin()

    @classmethod
    def get_point(cls):
        """
        Returns plugin point model instance. Only used from plugin classes.
        """
        if is_plugin_point(cls):
            raise Exception(_("This method is only available to plugin " "classes."))
        else:
            return cls.__base__

    @classmethod
    def get_point_model(cls):
        """
        Returns plugin point model instance. Only used from plugin classes.
        """
        if is_plugin_point(cls):
            raise Exception(_("This method is only available to plugin " "classes."))
        else:
            return PluginPointModel.objects.get(plugin__pythonpath=cls.get_pythonpath())

    @classmethod
    def get_plugins(cls):
        """
        Returns all plugin instances of plugin point, passing all args and
        kwargs to plugin constructor.
        """
        # Django >= 1.9 changed something with the migration logic causing
        # plugins to be executed before the corresponding database tables
        # exist. This method will only return something if the database
        # tables have already been created.
        if not db_table_exists(Plugin._meta.db_table):
            return

        if is_plugin_point(cls):
            for plugin_model in cls.get_plugins_qs():
                yield plugin_model.get_plugin()
        else:
            raise Exception(_("This method is only available to plugin point " "classes."))

    @classmethod
    def get_plugins_qs(cls):
        """
        Returns query set of all plugins belonging to plugin point.

        Example::

            for plugin_instance in MyPluginPoint.get_plugins_qs():
                print(plugin_instance.get_plugin().name)

        """
        if is_plugin_point(cls):
            point_pythonpath = cls.get_pythonpath()
            return Plugin.objects.filter(
                point__pythonpath=point_pythonpath, status=ENABLED
            ).order_by("index")
        else:
            raise Exception(_("This method is only available to plugin point " "classes."))

    @classmethod
    def get_name(cls):
        if is_plugin_point(cls):
            raise Exception(_("This method is only available to plugin " "classes."))
        else:
            return cls.get_model().name

    @classmethod
    def get_title(cls):
        if is_plugin_point(cls):
            raise Exception(_("This method is only available to plugin " "classes."))
        else:
            return cls.get_model().title



from __future__ import absolute_import, unicode_literals

from dirtyfields import DirtyFieldsMixin
from django.db import models
from django.utils.translation import ugettext_lazy as _

from djangoplugins.signals import django_plugin_disabled, django_plugin_enabled

from .utils import get_plugin_from_string, get_plugin_name

ENABLED = 0
DISABLED = 1
REMOVED = 2

STATUS_CHOICES = (
    (ENABLED, _("Enabled")),
    (DISABLED, _("Disabled")),
    (REMOVED, _("Removed")),
)

STATUS_CHOICES_ENABLED = (ENABLED,)
STATUS_CHOICES_DISABLED = (
    DISABLED,
    REMOVED,
)


class PluginPointManager(models.Manager):
    def get_point(self, point):
        return self.get(pythonpath=get_plugin_name(point))


class PluginPoint(models.Model):
    pythonpath = models.CharField(max_length=255)
    title = models.CharField(max_length=255)
    status = models.SmallIntegerField(choices=STATUS_CHOICES, default=ENABLED)

    objects = PluginPointManager()

    def __str__(self):
        return self.title


class PluginManager(models.Manager):
    def get_plugin(self, plugin):
        return self.get(pythonpath=get_plugin_name(plugin))

    def get_plugins_of(self, point):
        return self.filter(point__pythonpath=get_plugin_name(point), status=ENABLED)

    def get_by_natural_key(self, name):
        return self.get(pythonpath=name)


class Plugin(DirtyFieldsMixin, models.Model):
    """
    Database representation of a plugin.

    Fields ``name`` and ``title`` are synchronized from plugin classes.

    point
        Plugin point.

    pythonpath
        Full python path to plugin class, including class too.

    name
        Plugin slug name, must be unique within one plugin point.

    title
        Eny verbose title of this plugin.

    index
        Using values from this field plugins are orderd.

    status
        Plugin status.
    """

    point = models.ForeignKey(PluginPoint, on_delete=models.CASCADE)
    pythonpath = models.CharField(max_length=255, unique=True)
    name = models.CharField(max_length=255, null=True, blank=True)
    title = models.CharField(max_length=255, default="", blank=True)
    index = models.IntegerField(default=0)
    status = models.SmallIntegerField(choices=STATUS_CHOICES, default=ENABLED)

    objects = PluginManager()

    class Meta:
        unique_together = (("point", "name"),)
        ordering = ("index", "id")

    def __str__(self):
        if self.title:
            return self.title
        if self.name:
            return self.name
        return self.pythonpath

    def natural_key(self):
        return (self.pythonpath,)

    def is_active(self):
        return self.status == ENABLED

    def get_plugin(self):
        plugin_class = get_plugin_from_string(self.pythonpath)
        return plugin_class()

    def save(self, *args, **kwargs):
        if "status" in self.get_dirty_fields().keys() and self.pk:
            if self.status in STATUS_CHOICES_ENABLED:
                django_plugin_enabled.send(sender=self.__class__, plugin=self.get_plugin())
            else:
                django_plugin_disabled.send(sender=self.__class__, plugin=self.get_plugin())

        return super(Plugin, self).save(*args, **kwargs)



from django import forms
from django.db import models

from .models import Plugin
from .utils import get_plugin_name


class PluginField(models.ForeignKey):
    def __init__(self, point=None, *args, **kwargs):
        # If not migrating, add a new fields.
        if point is not None:
            kwargs["limit_choices_to"] = {"point__pythonpath": get_plugin_name(point)}

        super(PluginField, self).__init__(to=kwargs.pop("to", Plugin), *args, **kwargs)


class ManyPluginField(models.ManyToManyField):
    def __init__(self, point=None, *args, **kwargs):
        # If not migrating, add a new fields.
        if point is not None:
            kwargs["limit_choices_to"] = {"point__pythonpath": get_plugin_name(point)}

        super(ManyPluginField, self).__init__(to=kwargs.pop("to", Plugin), *args, **kwargs)


def get_plugins_qs(point):
    return point.get_plugins_qs().exclude(name__isnull=True)


class PluginChoiceField(forms.ModelChoiceField):
    def __init__(self, point, *args, **kwargs):
        kwargs["to_field_name"] = "name"
        super(PluginChoiceField, self).__init__(queryset=get_plugins_qs(point), **kwargs)

    def to_python(self, value):
        value = super(PluginChoiceField, self).to_python(value)
        if value:
            return value.get_plugin()
        else:
            return value


class PluginMultipleChoiceField(forms.ModelMultipleChoiceField):
    def __init__(self, point, *args, **kwargs):
        kwargs["to_field_name"] = "name"
        super(PluginMultipleChoiceField, self).__init__(queryset=get_plugins_qs(point), **kwargs)


class PluginModelChoiceField(forms.ModelChoiceField):
    def __init__(self, point, *args, **kwargs):
        super(PluginModelChoiceField, self).__init__(queryset=get_plugins_qs(point), **kwargs)


class PluginModelMultipleChoiceField(forms.ModelMultipleChoiceField):
    def __init__(self, point, *args, **kwargs):
        super(PluginModelMultipleChoiceField, self).__init__(
            queryset=get_plugins_qs(point), **kwargs
        )



# Generated by Django 3.2.20 on 2023-07-18 12:54

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("djangoplugins", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="plugin",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="pluginpoint",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]



from django import forms
from django.test import TestCase
from django.utils.translation import ugettext_lazy as _

from .fields import PluginChoiceField, PluginModelChoiceField, PluginModelMultipleChoiceField
from .management.commands.syncplugins import SyncPlugins
from .models import DISABLED, ENABLED, REMOVED, Plugin
from .models import PluginPoint as PluginPointModel
from .point import PluginMount, PluginPoint


class MyPluginPoint(PluginPoint):
    pass


class MyPlugin(MyPluginPoint):
    pass


class MyPluginFull(MyPluginPoint):
    name = "my-plugin-full"
    title = _("My Plugin Full")


class MyPlugin2(MyPluginPoint):
    name = "my-plugin-2"
    title = _("My Plugin 2")


class PluginSyncTestCaseBase(TestCase):
    def delete_plugins_from_db(self):
        Plugin.objects.all().delete()
        PluginPointModel.objects.all().delete()

    def prepate_query_sets(self):
        self.points = PluginPointModel.objects.filter(
            pythonpath="djangoplugins.tests.MyPluginPoint"
        )
        self.plugins = Plugin.objects.filter(pythonpath="djangoplugins.tests.MyPlugin")


class PluginSyncTestCase(PluginSyncTestCaseBase):
    def setUp(self):
        self.delete_plugins_from_db()
        self.prepate_query_sets()

    def test_plugins_not_synced(self):
        """
        At first there should not be any plugins in database
        """
        self.assertEqual(self.points.count(), 0)
        self.assertEqual(self.plugins.count(), 0)

    def test_plugins_are_synced(self):
        """
        Now sync plugins and check if they appear in database
        """
        SyncPlugins(False, 0).all()
        self.assertEqual(self.points.filter(status=ENABLED).count(), 1)
        self.assertEqual(self.plugins.filter(status=ENABLED).count(), 1)

    def test_plugins_meta(self):
        SyncPlugins(False, 0).all()
        plugin_model = MyPluginPoint.get_model("my-plugin-full")
        self.assertEqual("djangoplugins.tests.MyPluginFull", plugin_model.pythonpath)


class PluginSyncRemovedTestCase(PluginSyncTestCaseBase):
    def setUp(self):
        self.prepate_query_sets()
        self.copy_of_points = PluginMount.points

    def tearDown(self):
        PluginMount.points = self.copy_of_points

    def test_removed_plugins(self):
        """
        Remove all plugin points and sync again, without deleting.
        All plugin points (but not plugins) should be marked as REMOVED
        """
        PluginMount.points = []
        SyncPlugins(False, 0).all()
        self.assertEqual(self.points.filter(status=REMOVED).count(), 1)
        self.assertEqual(self.plugins.filter(status=REMOVED).count(), 0)

    def test_sync_and_delete(self):
        """
        Sync plugins again, with deleting all removed from database.
        Using cascaded deletes, all plugins, that belongs to being deleted
        plugin points will be deleted also.
        """
        PluginMount.points = []
        SyncPlugins(True, 0).all()
        self.assertEqual(self.points.count(), 0)
        self.assertEqual(self.plugins.count(), 0)


class PluginModelsTest(TestCase):
    def test_plugins_of_point(self):
        qs = MyPluginPoint.get_plugins_qs()
        self.assertEqual(3, qs.count())

    def test_plugin_model(self):
        plugin_name = "djangoplugins.tests.MyPlugin"
        plugin = Plugin.objects.get(pythonpath=plugin_name)
        self.assertEqual(plugin_name, str(plugin))
        self.assertEqual((plugin_name,), plugin.natural_key())

    def test_plugin_model_full(self):
        plugin_name = "djangoplugins.tests.MyPluginFull"
        plugin = Plugin.objects.get(pythonpath=plugin_name)
        self.assertEqual(_("My Plugin Full"), str(plugin))

    def test_plugin_point_model(self):
        point_name = "djangoplugins.tests.MyPluginPoint"
        point = PluginPointModel.objects.get(pythonpath=point_name)
        self.assertEqual("MyPluginPoint", str(point))

    def test_plugins_of_plugin(self):
        self.assertRaises(Exception, MyPlugin.get_plugins_qs)


class PluginsTest(TestCase):
    def test_get_model(self):
        point = "djangoplugins.tests.MyPluginPoint"
        plugin = "djangoplugins.tests.MyPluginFull"

        model = MyPluginFull.get_model()
        self.assertEqual(plugin, model.pythonpath)

        model = MyPluginFull().get_model()
        self.assertEqual(plugin, model.pythonpath)

        model = MyPluginPoint.get_model("my-plugin-full")
        self.assertEqual(plugin, model.pythonpath)

        model = MyPluginPoint.get_model()
        self.assertEqual(point, model.pythonpath)

        model = MyPluginPoint().get_model()
        self.assertEqual(point, model.pythonpath)

        model = MyPluginFull.get_point_model()
        self.assertEqual(point, model.pythonpath)

        self.assertRaises(Exception, MyPluginPoint.get_point_model)

    def test_get_point(self):
        point = MyPluginFull.get_point()
        self.assertTrue(point is MyPluginPoint)

        self.assertRaises(Exception, MyPluginPoint.get_point)

    def test_get_plugin(self):
        model = MyPluginFull.get_model()
        plugin = model.get_plugin()
        self.assertTrue(isinstance(plugin, MyPluginFull))

    def test_disabled_plugins(self):
        self.assertTrue(MyPluginFull.is_active())
        self.assertEqual(3, MyPluginPoint.get_plugins_qs().count())
        self.assertEqual(3, len(list(MyPluginPoint.get_plugins())))

        model = MyPluginFull.get_model()
        model.status = DISABLED
        model.save()

        self.assertFalse(MyPluginFull.is_active())
        self.assertEqual(2, MyPluginPoint.get_plugins_qs().count())
        self.assertEqual(2, len(list(MyPluginPoint.get_plugins())))
        self.assertRaises(Plugin.DoesNotExist, lambda: MyPluginPoint.get_model("my-plugin-full"))

        plugin_model = MyPluginPoint.get_model("my-plugin-full", status=None)
        self.assertEqual("my-plugin-full", plugin_model.name)

    def test_get_meta(self):
        self.assertEqual("my-plugin-full", MyPluginFull.get_name())
        self.assertEqual(_("My Plugin Full"), MyPluginFull.get_title())

        model = MyPluginFull.get_model()
        model.name = "test"
        model.save()

        self.assertEqual("test", MyPluginFull.get_name())


class MyTestForm(forms.Form):
    plugin_choice = PluginChoiceField(MyPluginPoint)
    model_choice = PluginModelChoiceField(MyPluginPoint)

    # plugin_multi_choice = PluginMultipleChoiceField(MyPluginPoint)
    model_multi_choice = PluginModelMultipleChoiceField(MyPluginPoint)


class PluginsFieldsTest(TestCase):
    def test_validation(self):
        form = MyTestForm(
            {
                "plugin_choice": "my-plugin-2",
                "model_choice": "%d" % MyPlugin2.get_model().id,
                # 'plugin_multi_choice': ['my-plugin-2'],
                "model_multi_choice": ["%d" % MyPlugin2.get_model().id],
            }
        )
        self.assertTrue(form.is_valid())

        cld = form.cleaned_data
        self.assertTrue(isinstance(cld["plugin_choice"], MyPlugin2))
        self.assertTrue(isinstance(cld["model_choice"], Plugin))
        self.assertTrue(isinstance(cld["model_multi_choice"][0], Plugin))



from __future__ import absolute_import, unicode_literals

from django.core.management.base import BaseCommand

from djangoplugins.models import ENABLED, REMOVED, Plugin, PluginPoint
from djangoplugins.point import PluginMount
from djangoplugins.utils import db_table_exists, get_plugin_name, load_plugins


class Command(BaseCommand):
    help = "Syncs the registered plugins and plugin points with the model " "versions."
    requires_model_validation = True

    def add_arguments(self, parser):
        parser.add_argument(
            "--delete",
            action="store_true",
            dest="delete",
            help="delete the REMOVED Plugin and PluginPoint " "instances. ",
        )

    def handle(self, *args, **options):
        sync = SyncPlugins(options.get("delete"), options.get("verbosity"))
        sync.all()


class SyncPlugins:
    """
    In most methods ``src`` and ``dst`` variables are used, they meaning is:

    ``src``
        source, registered plugin point objects

    ``dst``
        destination, database
    """

    def __init__(self, delete_removed=False, verbosity=1):
        load_plugins()
        self.delete_removed = delete_removed
        self.verbosity = int(verbosity)

    def print_(self, verbosity, message):
        if self.verbosity >= verbosity:
            print(message)

    def get_classes_dict(self, classes):
        return dict([(get_plugin_name(i), i) for i in classes])

    def get_instances_dict(self, qs):
        return dict((i.pythonpath, i) for i in qs)

    def available(self, src, dst, model):
        """
        Iterate over all registered plugins or plugin points and prepare to add
        them to database.
        """
        for name, point in iter(src.items()):
            inst = dst.pop(name, None)
            if inst is None:
                self.print_(1, "Registering %s for %s" % (model.__name__, name))
                inst = model(pythonpath=name)
            if inst.status == REMOVED:
                self.print_(1, "Updating %s for %s" % (model.__name__, name))
                # re-enable a previously removed plugin point and its plugins
                inst.status = ENABLED
            yield point, inst

    def missing(self, dst):
        """
        Mark all missing plugins, that exists in database, but are not
        registered.
        """
        for inst in iter(dst.values()):
            if inst.status != REMOVED:
                inst.status = REMOVED
                inst.save()

    def delete(self, dst):
        count = dst.objects.filter(status=REMOVED).count()
        if count:
            self.print_(1, "Deleting %d Removed %ss" % (count, dst.__name__))
            dst.objects.filter(status=REMOVED).delete()

    def points(self):
        src = self.get_classes_dict(PluginMount.points)
        dst = self.get_instances_dict(PluginPoint.objects.all())

        for point, inst in self.available(src, dst, PluginPoint):
            if hasattr(point, "_title"):
                inst.title = point._title
            else:
                inst.title = inst.pythonpath.split(".")[-1]
            inst.save()
            self.plugins(point, inst)

        self.missing(dst)

        if self.delete_removed:
            self.delete(PluginPoint)

    def plugins(self, point, point_inst):
        src = self.get_classes_dict(point.plugins)
        dst = self.get_instances_dict(point_inst.plugin_set.all())

        for plugin, inst in self.available(src, dst, Plugin):
            inst.point = point_inst
            inst.name = getattr(plugin, "name", None)
            if hasattr(plugin, "title"):
                inst.title = getattr(plugin, "title")
            inst.save()

        self.missing(dst)

    def all(self):
        """
        Synchronize all registered plugins and plugin points to database.
        """
        # Django >= 1.9 changed something with the migration logic causing
        # plugins to be executed before the corresponding database tables
        # exist. This method will only return something if the database
        # tables have already been created.
        if not db_table_exists(Plugin._meta.db_table) or not db_table_exists(
            PluginPoint._meta.db_table
        ):
            return
        self.points()



from django.template import Library, Node, TemplateSyntaxError

from ..utils import get_plugin_from_string

register = Library()


class PluginsNode(Node):
    def __init__(self, point_name, var_name):
        self.plugins = get_plugin_from_string(point_name).get_plugins()
        self.var_name = var_name

    def render(self, context):
        context[self.var_name] = self.plugins
        return ""


@register.tag
def get_plugins(parser, token):
    contents = token.split_contents()
    if len(contents) != 4:
        raise TemplateSyntaxError("%r tag requires exactly 3 arguments" % (contents[0]))
    if "as" != contents[2]:
        raise TemplateSyntaxError("%r tag 2nd argument must be 'as'" % (contents[0]))
    return PluginsNode(contents[1], contents[3])



from importlib import import_module
import sys

from django.conf import settings
from django.conf.urls import include, url
from django.db import connection


def get_plugin_name(cls):
    return "%s.%s" % (cls.__module__, cls.__name__)


def get_plugin_from_string(plugin_name):
    """
    Returns plugin or plugin point class from given ``plugin_name`` string.

    Example of ``plugin_name``::

        'my_app.MyPlugin'

    """
    modulename, classname = plugin_name.rsplit(".", 1)
    module = import_module(modulename)
    return getattr(module, classname)


def include_plugins(point, pattern=r"{plugin}/", urls="urls"):
    # This hack allows us to run a syncplugins without including plugins in urls.
    if sys.argv[0] == "manage.py" and sys.argv[1] in ("migrate", "makemigration"):
        return include([])

    pluginurls = []
    for plugin in point.get_plugins():
        if hasattr(plugin, urls) and hasattr(plugin, "name"):
            _urls = getattr(plugin, urls)
            for _url in _urls:
                _url.default_args["plugin"] = plugin.name
            pluginurls.append(url(pattern.format(plugin=plugin.name), include(_urls)))
    return include(pluginurls)


def import_app(app_name):
    try:
        mod = import_module(app_name)
    except ImportError:  # Maybe it's AppConfig
        parts = app_name.split(".")
        tmp_app, app_cfg_name = ".".join(parts[:-1]), parts[-1]
        try:
            tmp_app = import_module(tmp_app)
        except ImportError:
            raise
        mod = getattr(tmp_app, app_cfg_name).name

        # Workaround for not finding app plugins modules in all cases
        try:
            mod = import_module("{}.plugins".format(mod))
        except ImportError:
            pass

    return mod


def load_plugins():
    for app in settings.INSTALLED_APPS:
        try:
            import_module("%s.plugins" % app)
        except ImportError:
            import_app(app)


def db_table_exists(table_name):
    return table_name in connection.introspection.table_names()



# Import cascade to work with a number of Django versions.
try:
    from django.db.models.signals import post_migrate
except ImportError:
    try:
        from django.db.models.signals import post_syncdb as post_migrate
    except ImportError:
        from south.signals import post_migrate

from djangoplugins import models as plugins_app

from .commands.syncplugins import SyncPlugins


def sync_plugins(sender, verbosity, **kwargs):
    # Different django version have different senders.
    if (hasattr(sender, "name") and sender.name == "djangoplugins") or (sender == plugins_app):
        SyncPlugins(False, verbosity).all()


# Plugins must be synced to the database.
post_migrate.connect(sync_plugins)



from django.contrib import admin

from .models import Plugin


class PluginAdmin(admin.ModelAdmin):
    list_display = ("title", "index", "status")
    list_filter = ("point", "status")


admin.site.register(Plugin, PluginAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import dirtyfields.dirtyfields
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Plugin",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID", serialize=False, primary_key=True, auto_created=True
                    ),
                ),
                ("pythonpath", models.CharField(unique=True, max_length=255)),
                ("name", models.CharField(max_length=255, blank=True, null=True)),
                ("title", models.CharField(max_length=255, blank=True, default="")),
                ("index", models.IntegerField(default=0)),
                (
                    "status",
                    models.SmallIntegerField(
                        choices=[(0, "Enabled"), (1, "Disabled"), (2, "Removed")], default=0
                    ),
                ),
            ],
            options={
                "ordering": ("index", "id"),
            },
            bases=(dirtyfields.dirtyfields.DirtyFieldsMixin, models.Model),
        ),
        migrations.CreateModel(
            name="PluginPoint",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID", serialize=False, primary_key=True, auto_created=True
                    ),
                ),
                ("pythonpath", models.CharField(max_length=255)),
                ("title", models.CharField(max_length=255)),
                (
                    "status",
                    models.SmallIntegerField(
                        choices=[(0, "Enabled"), (1, "Disabled"), (2, "Removed")], default=0
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="plugin",
            name="point",
            field=models.ForeignKey(to="djangoplugins.PluginPoint", on_delete=models.CASCADE),
        ),
        migrations.AlterUniqueTogether(
            name="plugin",
            unique_together=set([("point", "name")]),
        ),
    ]



import os.path

from setuptools import find_packages, setup


def read_docs(filename):
    path = os.path.join(os.path.dirname(__file__), filename)
    return open(path).read()


long_description = """
``django-plugins`` offers functionality to make Django apps them more reusable.

For maintained by @bihealth, aims at Django >=3.0.

Home page
    http://pypi.python.org/pypi/django-plugins-bihealth

Source code:
    https://github.com/bihealth/django-plugins\n\n""".lstrip()

setup(
    name="django-plugins-bihealth",
    version="0.4.0",
    author="Mikko Nieminen, Manuel Holtgrewe, Oliver Stolpe",
    author_email="mikko.nieminen@bih-charite.de, manuel.holtgrewe@bih-charite.de, oliver.stolpe@bih-charite.de",
    packages=find_packages(exclude=["sample-project"]),
    install_requires=[
        "django>=3.0",
        "django-dirtyfields",
    ],
    url="https://github.com/bihealth/django-plugins",
    download_url="http://pypi.python.org/pypi/django-plugins-bihealth",
    license="LGPL",
    description="django-plugins-bihealth",
    long_description=long_description,
    include_package_data=True,
    exclude_package_data={"": ["sample-project"]},
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: " "GNU Library or Lesser General Public License (LGPL)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
